﻿// Network Table values:
//
//      thunderdashboard_breacher       2016 - breacher position form 0 to 100
//      thunderdashboard_scalerpivot    2016 - arm's position from 0 to 100
//      thunderdashboard_scalerext      2016 - arm's extension from 0 to 100
//      thunderdashboard_armlocked      arm is locked into position (used in: 2016)


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
    public partial class Form2016: FormBase
    {
        public Form2016()
        {
            InitializeComponent();

            Text = "Sir Rolling of Thunder"; // Robot name goes here (i.e. tab's text)
            DebugTitle = "Zack and Kate's Savior"; // title for the debug tab

            // load controller images
            ControllerAux = LoadImage("2016", "ControlsAux.png");
            ControllerDriver = LoadImage("2016", "ControlsDriver.png");

            // optional tabs
            TabLightsPsoc42xx = true;   // show the light tab that talks to the PSOC 42xx via a COM port?
            TabChart = false;            // show the chart tab?



            // *******************************************************************************
            // any custom control settings for this year's robot

            //
            // *******************************************************************************



            // *******************************************************************************
            // NOTE: if a control/feature is not supported (i.e. control does not exist), then
            //       set the control to null (e.g. CameraFeed = null;)
            // *******************************************************************************

            // camera
            CameraPort = 1181; // port the camera feed is on (as of 2018 it is MPEJ on 1181, previous roboRIO years it was 1180)
            CameraFlip = false; // camera is mounted upside down, so flip image before showing it?
            CameraFeed = cameraControl; // control that gets the camera feed
            CameraVisionDelegate = null; // method to call if there is any vision processing that the dashboard will do
            CameraFeedDirection = labelCameraSide; // indicates if camera feed is from the front or rear of robot (only needed is two cameras on robot)

            // drive
            RobotDirection = labelReverseDrive; // will so REVERSE when driver controls have been swapped so rear of robot is the 'front'

            // auto mode
            AutoModeComboBox = comboBoxAutoMode; // gets list of auto modes for user to select from
            AutoModeDescriptionsTextBox = autoModeDescriptionsBox; // gets the description of the auto mode (not used in 2017)
            AutoModeFromRobot = labelAutoModeValue; // auto mode the robot currently has selected

            // connection/status
            PanelConnected = panelConnected; // state of connection with network table
            StatusTextBox = statusTextBox; // any status messages about connection and images
        }

        // called with Reset button is pressed
        public override void buttonReset_Click(object sender, EventArgs e)
        {
            // just let base class to it all
            base.buttonReset_Click(sender, e);
        }

        // called when the network table is updated
        //
        // returns true if the form handled the key
        public override bool NetworkTableUpdated(string key, string value)
        {
            switch (key)
            {
                default:
                    return (base.NetworkTableUpdated(key, value));

                case "thunderdashboard_breacher":
                    meterBreacherPosition.Value = value;
                    break;

                case "thunderdashboard_scalerpivot":
                    meterScalerPivot.Value = value;
                    break;

                case "thunderdashboard_scalerext":
                    meterScalerExtension.Value = value;
                    break;

                case "thunderdashboard_armlocked":
                    // sees if scaler arm is locked and changes color based on that
                    if (value == "1")
                    {
                        meterScalerPivot.Color(Color.Red);
                    }
                    else
                    {
                        meterScalerPivot.ColorReset();
                    }
                    break;
            }

            return (false);
        }
    }
}
