﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
	public partial class FormMain
	{
        private System.Diagnostics.Process _serverProcess = null;

        // stop the network table if we kicked it off
        public void NetworkTableServerStop()
        {
            if ((_serverProcess != null) && !_serverProcess.HasExited)
            {
                _serverProcess.CloseMainWindow();
            }
        }

        // do it all: launch another dashobard to be the 'robot', and change us to connect to that
        private void buttonNetTblStart_Click(object sender, EventArgs e)
        {
            // start a second instance of ourselves  and tell it to run as a Server
            _serverProcess = new System.Diagnostics.Process();
            string file = Environment.GetCommandLineArgs()[0];// System.Reflection.Assembly.GetEntryAssembly().GetName().CodeBase;
            _serverProcess.StartInfo = new System.Diagnostics.ProcessStartInfo(file, "-RunAsServer");
            if (!_serverProcess.Start())
            {
                MessageBox.Show("Unable to start another dashboard to act like the robot");
                return;
            }

            // since we have started the other dashboard, now connect us to that table instead
            buttonClient_Click(sender, e);
        }

        // Server button was pressed
        public void buttonServer_Click(object sender, EventArgs e)
		{
            // start the network table server
            DashboardNetworkTable.Start("", null, false);

            // don't let user play with this buttons anymore, because they will do nothing/break things
            buttonNetTblStart.Enabled = false;
            buttonServer.Enabled = false;
            buttonClient.Enabled = false;

            Text += " - Running as robot"; // update caption to reflect what just happened
            tabControlDashboard.Visible = false; // hide all the tabs so the user doens't try to do anything stupid
            WindowState = FormWindowState.Minimized; // get our window off the screen
        }

        // Client button was pressed
        private void buttonClient_Click(object sender, EventArgs e)
		{
            // connect to the network table on this computer
            DashboardNetworkTable.Start("127.0.0.1", NetworkTableDelegate);

            // don't let user play with this buttons anymore, because they will do nothing/break things
            buttonNetTblStart.Enabled = false;
            buttonServer.Enabled = false;
			buttonClient.Enabled = false;

            Text += " - Using PC, NOT robot"; // update caption to reflect what just happened
        }

        // add a string to the network table
        private void buttonStringAdd_Click(object sender, EventArgs e)
		{
			if (!DashboardNetworkTable.PutString(textBoxStringName.Text, textBoxStringValue.Text))
			{
				MessageBox.Show(DashboardNetworkTable.LastError(), "ERROR");
			}
		}

		// get a string to the network table
		private void buttonStringGet_Click(object sender, EventArgs e)
		{
			string value = "";
			if (!DashboardNetworkTable.GetString(textBoxStringName.Text, ref value, "empty"))
			{
				MessageBox.Show(DashboardNetworkTable.LastError(), "ERROR");
			}
			textBoxStringValue.Text = value;
		}

        private void buttonNumberAdd_Click(object sender, EventArgs e)
        {
            double value;
            try
            {
                value = Convert.ToDouble(textBoxNumberValue.Text);
            }
            catch
            {
                MessageBox.Show("Value is not a number", "ERROR");
                return;
            }
            if (!DashboardNetworkTable.PutNumber(textBoxNumberName.Text, value))
            {
                MessageBox.Show(DashboardNetworkTable.LastError(), "ERROR");
            }
        }

        private void buttonNumberGet_Click(object sender, EventArgs e)
        {
            double value = 0;
            if (!DashboardNetworkTable.GetNumber(textBoxNumberName.Text, ref value, 0))
            {
                MessageBox.Show(DashboardNetworkTable.LastError(), "ERROR");
            }
            textBoxNumberValue.Text = value.ToString();
        }
   
        // called when table is changed
        private void NetworkTableDelegateDebug(string key, string value)
		{
			if (key == textBoxStringName.Text)
			{
				textBoxStringValue.Text = value;
			}
		}
	}
}
