﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThunderDashboard
{
    public partial class FormNetTblEdit : Form
    {
        public FormNetTblEdit(string keyName, string valueString, bool isString)
        {
            InitializeComponent();
            labelKeyName.Text = keyName + ":";
            textBoxValue.Text = valueString;

            _isString = isString;
            if (!isString)
            {
                textBoxValue.TextChanged += TextBoxValue_TextChanged;
            }
        }

        // get the value the user entered
        public string GetValueString
        {
            get { return (textBoxValue.Text); }
        }

        // get the value the user entered as a number
        public double GetValueNumber
        {
            get
            {
                try
                {
                    return (Convert.ToDouble(textBoxValue.Text));
                }
                catch
                {
                    return (0);
                }
            }
        }

        // handle 'Set' being clicked
        private void buttonOK_Click(object sender, EventArgs e)
        {
            // if it is a number, then make sure it is valid
            if (!_isString)
            {
                TextBoxValue_TextChanged(null, null);
            }
        }

        // valid the text to make sure it is a number
        private void TextBoxValue_TextChanged(object sender, EventArgs e)
        {
            string text = textBoxValue.Text;
            string textNew = "";

            if (text.Length <= 0)
            {
                return;
            }

            for (int i = 0; i < text.Length; i++)
            {
                switch (text[i])
                {
                    default:            // invalid
                        break;

                    case '-':           // can only be at first position
                        if (i == 0)
                        {
                            textNew += text[i];
                        }
                        break;

                    case '.':           // can only have one
                        if (textNew.IndexOf('.') < 0)
                        {
                            textNew += text[i];
                        }
                        break;

                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        textNew += text[i];
                        break;
                }
            }

            // if the text changed (i.e. ws in valid) then update the control
            if (text != textNew)
            {
                textBoxValue.Text = textNew;
                textBoxValue.Select(textBoxValue.Text.Length, 0);
            }
        }

        private bool _isString;
    }
}
