﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ThunderDashboard
{
    class MeterControl : Panel
    {
        // meter's value
        public string Value
        {
            get { return _value.ToString(); }
            set
            {
                // if string is invalide, then set to minimum
                if (string.IsNullOrEmpty(value))
                {
                    SetValue(_minValue);
                }
                else
                {
                    try
                    {
                        SetValue(Convert.ToDouble(value));
                    }
                    catch
                    {
                        SetValue(_minValue);
                    }
                }
            }
        }

        // set the decimal places for the number within the gauge
        public int ThunderDecimalPlaces
        {
            get { return (_decimalPlaces); }
            set
            {
                if (value != _decimalPlaces)
                {
                    _decimalPlaces = value;
                    Redraw();
                }
            }
        }

        // is the meter gauge drawn horizontally
        public bool ThunderMeterHorizontal
        {
            get { return _meterHorizontal; }
            set { _meterHorizontal = value; }
        }

        // is the meter drawn left-to-right (i.e. horizontal)
        public bool ThunderMeterLeftToRight
        {
            get { return _meterLeftToRight; }
            set { _meterLeftToRight = value; }
        }





        // set the color for the meter (set to NULL to use default color)
        public void Color(System.Drawing.Color color)
		{
			// resetting color?
			if (color == null)
			{
				color = _colorDefault;
			}

			// if color changed, then save it and re-draw the meter
			if (color != _color)
			{
				_color = color;
				Redraw();
			}
		}

        // will reset the color to the default
        public void ColorReset()
        {
            Color(_colorDefault);
        }

        // set the Min and Max values that could ever be sent (i.e. bottom and top value for the guage)
        public void MinMax(double min, double max)
		{
			double value = _value;

			if (min > max)
			{
				min = max;
			}
			if (_value > max)
			{
				_value = max;
			}
			if (_value < min)
			{
				_value = min;
			}
			if ((min != _minValue) || (max != _maxValue) || (value != _value))
			{
				_minValue = min;
				_maxValue = max;
				_value = value;
				Redraw();
			}
		}

        protected override void OnClick(EventArgs e)
        {
            base.OnClick(e);
            //int xx = Height - PointToClient(Cursor.Position).Y;
            //Value = ((PointToClient(Cursor.Position).Y - _minValue) * Height / (_maxValue - _minValue)).ToString();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            DrawFillRectangle();
        }

		// force control to be redrawn
		private void Redraw()
		{
			// This repaints the control. Per MSDN: Calling the Invalidate method does not force a synchronous paint;
			// to force a synchronous paint, call the Update method after calling the Invalidate method.
			this.Invalidate();
			this.Update();
		}

		private void DrawFillRectangle()
        {
            // Draws the green rectangle indicating the value set.
            System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(_color);
            System.Drawing.Graphics formGraphics;
            formGraphics = this.CreateGraphics();

            int width = 0;
            int height = 0;
            int x = 0;
            int y = 0;
            double percent = ((double)_value - (double)_minValue) / ((double)_maxValue - (double)_minValue);
            if (ThunderMeterHorizontal)
            {
                height = this.Height;
                // Width should be a percentage of control 
                width = (int)(this.Width * percent);

                if (!ThunderMeterLeftToRight)
                {
                    x = this.Width - width; // Need to change the x position of the rectangle to be drawn
                }
            }
            else // vertical layout
            {
                width = this.Width;
                // Height should be a percentage of control;
                height = (int)(this.Height * percent);

                y = this.Height - height;
            }

            formGraphics.FillRectangle(myBrush, new Rectangle(x, y, width, height));
            myBrush.Dispose();

            StringFormat f = new StringFormat();
            f.LineAlignment = StringAlignment.Center;
            f.Alignment = StringAlignment.Center;

            double d = 100.0 * percent;
            string s = string.Format("{0:N" + _decimalPlaces + "}", d);
            formGraphics.DrawString(s, new Font("Arial", 18), Brushes.Black, this.ClientRectangle, f);
            formGraphics.Dispose();
        }

        // make sure the value stays within the Min/Max range
        private void SetValue(double value)
        {
            // make sure value is within range
            if (value > _maxValue)
            {
                value = _maxValue;
            }
            if (value < _minValue)
            {
                value = _minValue;
            }

            // if the value has changed, then save it and redraw meter
            if (_value != value)
            {
                _value = value;
                Redraw();
            }
        }

        // color for the meter
        private static System.Drawing.Color _colorDefault = System.Drawing.Color.Lime;
		private System.Drawing.Color _color = _colorDefault;

		// how to draw the meter
		private bool _meterHorizontal = true;
		private bool _meterLeftToRight = true;

		// current value
		private double _value = 0;
		private double _maxValue = 100;
		private double _minValue = 0;
        private int _decimalPlaces = 0;
	}
}
