//MAB todo
// width is messed up when box is on right and can't see the right (We should skip those)
// maybe skip any box that doesn't have all edges visible; can't tell side anyway
//
// track which one is in the center
// for closest to center:
//       if row around center, use it
//       else pick row closet to center
//		do same for target within row
////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	The order for processing an image:
//		1. ConvertRgbToMono
//			- looks at given RGBValue image
//			- fills in monoImage
//			- fills in histogram with num pixels in each row of the monoImage
//			- for PC_TEST_APP, with erase background from given RGBValue image
//
//		2. CalcTargetRows
//			- looks at histogram
//			- fills in rowInfo[], but skips rows without enough pixels
//
//		3. CalcTargetColumns
//			- calls CreateRowHisto() and then FindTargets() for each row
//				CreateRowHisto():
//					- looks at monoImage and rowInfo[]
//					- fills in histogram with num pixel in each column within given row
//				FindTargets():
//					- looks at historgram and rowInfo[]
//					- fills in targetInfo[], will 'ignore' gaps ('merge' targets close together) and skip ones without enough pixels
//					- for PC_TEST_APP, fills in targetInfoTest[] and targetInfoTestG[]
//
//		4. CalcTargets
//			- calls FineTune() and CalcHeightAndTopEdge() for each target
//				FineTure():
//					- looks at targetInfo[] and monoImage
//					- uses historgram internally
//					- fills in targetInfo[].maxWidth/avgWidth
//					- fills in 'skew' of left edge
//					- updates targettInfo[].left/right/top/bottom to fit target exactly
//				CalcHeightAndTopEdge();
//					- looks at targetInfo[] and monoImage
//					- uses historgram internally
//					- fills in targetInfo[].maxHeight/avgHeight
//					- fills in 'skew' of top edge
//
////////////////////////////////////////////////////////////////////////////////////////////////////
#if CPP
//#include "thundervision.h"
#endif //CPP

#if !CPP
#define PC_TEST_APP
using System.Drawing;

namespace ThunderDashboard
{
    partial class ThunderVision
    {
#endif

#if CPP
        ThunderVision::ThunderVision()
#else
        public ThunderVision()
#endif
        {
            Init();
        }

#if CPP
ThunderVision::~ThunderVision()
{
}
#endif

        // initialize member variables
#if CPP
void ThunderVision::Init()
#else
        private unsafe void memcpy(byte *dst, byte *src, int numBytes)
        {
            for (int i = 0; i < numBytes; i++)
            {
                *dst = *src;
                dst++;
                src++;
            }
        }
        private void ClearArray(byte[] buffer)
        {
            for (int i = 0; i < buffer.Length; i++)
            {
                buffer[i] = 0;
            }
        }
        private void ClearArray(int[] buffer)
        {
            for (int i = 0; i < buffer.Length; i++)
            {
                buffer[i] = 0;
            }
        }
        private void Init()
#endif
        {
            //memset(histogram, 0, sizeof(histogram));
            ClearArray(histogram);
            //memset(monoImage, 0, sizeof(monoImage));
            ClearArray(monoImage);
            targets = 0;
            rows = 0;

            _imageWidth = 640;
            _imageHeight = 480;
            _pixelPerRow = MIN_PIXELS_ROW_640;
            _minWidth = MIN_WIDTH_640;
            _minHeight = MIN_HEIGHT_640;

            // for PC test app only
#if PC_TEST_APP
            targetsTest = 0;
            targetsTestG = 0;
#endif // PC_TEST_APP
        }

        // will process the entire image all at once (i.e. run STEPs 1 through 4)
#if CPP
        int ThunderVision::ProcessImage(RGBValue *image, int width, int height)
#else
        public unsafe int ProcessImage(RGBValue* image, int width, int height)
#endif
        {
            if (!ConvertRgbToMono(image, width, height))// STEP 1
            {
                return (0);
            }
            CalcTargetRows();                           // STEP 2
            CalcTargetColumns();                        // STEP 3
            CalcTargets();                              // STEP 4
            return (targets);
        }

        // convert the given RGB camera image to a monochrome image (i.e. pull out the green)
#if CPP
    bool ThunderVision::ConvertRgbToMono(RGBValue *image, int width, int height)
#else
        public unsafe bool ConvertRgbToMono(RGBValue* image, int width, int height)
#endif
        {
            byte* mono;
            int x;
            int y;

            Init();

            // can only handle 640x40 or 320x240
            if ((width < 320) || (height < 240) || (width > 640) || (height > 480))
            {
                return (false);
            }

            if (width == 320)
            {
                _pixelPerRow = MIN_PIXELS_ROW_320;
                _minWidth = MIN_WIDTH_320;
                _minHeight = MIN_HEIGHT_320;
            }
            else
            {
                _pixelPerRow = MIN_PIXELS_ROW_640;
                _minWidth = MIN_WIDTH_640;
                _minHeight = MIN_HEIGHT_640;
            }
            _imageWidth = width;
            _imageHeight = height;

            fixed (byte* monoptr = &monoImage[0])
            {
                // process each row
                mono = monoptr;
                for (y = 0; y < _imageHeight; y++)
                {
                    // process each pixel in the row
                    for (x = 0; x < _imageWidth; x++, mono++, image++)
                    {
                        if (   (image->G >= GREEN_THRESHOLD)            // green bright enough
                            && (image->R < (image->G - RED_DELTA))      // red far enough from green (need bluish-green)
                            && (image->B >= BLUE_THRESHOLD)             // blue bright enough (stay away from browns/blacks)
                            && (image->B <= (image->G + BLUE_DELTA))    // blue not too much brigther than green (stay away from blues)
                            )
                        {
                            *mono = 255;
                            histogram[y]++;                             // keep track of row that has a pixel
                        }
#if PC_TEST_APP
                        else
                        {
                            // for test app, wipe out the background
                            image->R = image->G = image->B = 0;
                        }
#endif // PC_TEST_APP
                    }
                }
            }
            return (true);
        }

        // look at each row to see if it has any pixels in it.
        // any successive lines (with pixels) will be considered part of the same target
        //string info = "";
#if CPP
    void ThunderVision::CalcTargetRows()
#else
        public unsafe void CalcTargetRows()
#endif
        {
            int y;
            int i;
            int* histo;

            // look at each row
            rows = 0;                   // no rows yet
            y = 0;                      // start at first row
            //info = "";
            fixed (int* histoptr = &histogram[0])          // start at first row (using a pointer is faster at runtime)
            {
                histo = histoptr;
                //info += string.Format("{0},{1}\r\n", _pixelPerRow, _minHeight);
                while ((y < _imageHeight) && (rows < MAX_ROWS))
                {
                    // find first pixel
                    while ((y < _imageHeight) && (*histo == 0))
                    {
                        y++;
                        histo++;
                    }
                    if (y == _imageHeight)                  // make sure we found a pixel. if not, go to next row
                    {
                        break;
                    }
                    rowInfo[rows].top = y;          // remember top-most row that has pixels

                    // find the last pixel and keep count of how many total pixels there are
                    i = 0;
                    while (y < _imageHeight && (*histo != 0))
                    {
                        i += *histo;
                        y++;
                        histo++;
                    }
                    rowInfo[rows].bottom = (y - 1); // remember bottom-most row that has pixels

                    // to help get rid of noise and diagonal line, we will only keep
                    // this set if the average number of pixels is high enough
                    // AND row is height enough
                    //info += string.Format("{0},{1} {2} {3}\n\n", (i / (rowInfo[rows].bottom - rowInfo[rows].top + 1)), (rowInfo[rows].bottom - rowInfo[rows].top + 1),
                    //    (((i / (rowInfo[rows].bottom - rowInfo[rows].top + 1)) > 5) && ((rowInfo[rows].bottom - rowInfo[rows].top + 1) >= 20)),
                    //    (((i / (rowInfo[rows].bottom - rowInfo[rows].top + 1)) > 3) && ((rowInfo[rows].bottom - rowInfo[rows].top + 1) >= 15)));
                    if (((i / (rowInfo[rows].bottom - rowInfo[rows].top + 1)) > _pixelPerRow)
                        && ((rowInfo[rows].bottom - rowInfo[rows].top + 1) >= _minHeight))
                    {
                        rows++;
                    }
                }
                //System.Windows.Forms.MessageBox.Show(info, "Rows");
            }
        }

        // look at each row (from CalcTargetRows()) and find all each box
#if CPP
void ThunderVision::CalcTargetColumns()
#else
        public void CalcTargetColumns()
#endif
        {
            int row;

            targets = 0;                    // no targets yet
#if PC_TEST_APP
            targetsTest = 0;
            targetsTestG = 0;
#endif // PC_TEST_APP

            // process each target row
            for (row = 0; row < rows; row++)
            {
                // create column histogram for the target row
                CreateRowHisto(row);

                // add targets to end of list
                // use 1/2 the height of the row as the amount of allowed 'gap' (i.e. ball/hoop)
                FindTargets(targets, row);
            }
        }

        // look at each target from CalcTargetColumns() and 'fine tune' it and get final values
#if CPP
void ThunderVision::CalcTargets()
#else
        public unsafe void CalcTargets()
#endif
        {
            int target;

            for (target = 0; target < targets; target++)
            {
                fixed (TARGET_INFO* ti = &targetInfo[target])
                {
                    FineTune(ti);
                    CalcHeightAndTopEdge(ti);
                }
            }
        }

#if CPP
void ThunderVision::FineTune(TARGET_INFO *targetInfo)
#else
        private unsafe void FineTune(TARGET_INFO* targetInfo)
#endif
        {
            int x;
            int y;
            int left;
            int leftTmp;
            int right;
            int rightTmp;
            int top;
            int bottom;
            bool found = false;
            byte* mono;

            //memset(histogram, 0, sizeof(histogram));
            ClearArray(histogram);
            targetInfo->avgWidth = 0;
            targetInfo->maxWidth = 0;
            left = 641;
            right = 0;
            top = -1;
            bottom = 0;
            for (y = targetInfo->top; y <= targetInfo->bottom; y++)
            {
                leftTmp = 641;                  // track left-most pixel in this row
                rightTmp = 0;                   // track right-most pixel in this row
                found = false;                  // no pixels in this row yet
                fixed (byte *monoptr = &monoImage[(y * _imageWidth) + targetInfo->left])
                {
                    mono = monoptr;
                    for (x = targetInfo->left; x <= targetInfo->right; x++, mono++)
                    {
                        // if pixel is set, then update tracking variables
                        if (*mono != 0)
                        {
                            if (!found)
                            {
                                // it is the first found pixel, so save as left-most in this row
                                leftTmp = x;
                                histogram[y] = x;
                            }
                            rightTmp = x;           // any pixel with end up being the right-most pixel
                            found = true;           // at least one pixel
                        }
                    }
                }

                // if we found at least one pixel, then update overall tracking values
                if (found)
                {
                    if (leftTmp < left)         // if this row's left is more left that prior row's, save it
                    {
                        left = leftTmp;
                    }
                    if (rightTmp > right)       // if this row's right is more right that prior row's, save it
                    {
                        right = rightTmp;
                    }
                    if (top == -1)              // if top not set yet, then save it
                    {
                        top = y;
                    }
                    bottom = y;                 // any pixel in row ends up being the bottom-most row

                    // keep track of max width and total width (so we can do average)
                    if ((rightTmp - leftTmp + 1) > targetInfo->maxWidth)
                    {
                        targetInfo->maxWidth = (rightTmp - leftTmp + 1);
                    }
                    targetInfo->avgWidth += (rightTmp - leftTmp + 1);
                }
            }

            // save updated values
            targetInfo->left = left;
            targetInfo->right = right;
            targetInfo->top = top;
            targetInfo->bottom = bottom;
            targetInfo->avgWidth /= (bottom - top + 1);

            // determine the 'skew' of the left edge
            // we only want to look at the middle of the image (to avoid the angles of the top & bottom)
            x = 0;
            y = (targetInfo->bottom - targetInfo->top + 1) >> 2;    // get 1/4 of the total height
            rightTmp = targetInfo->bottom - y;                      // end 1/4 from the bottom
            y += targetInfo->top;                                   // start 1/4 from the top
            leftTmp = histogram[y];                                 // initialize with first value
            for (; y <= rightTmp; y++)
            {
                // if value is greater than the previous, then the line
                // going down; otherise the line is going up
                if (histogram[y] > leftTmp)
                {
                    x--;                            // moved down
                }
                else if (histogram[y] < leftTmp)
                {
                    x++;                            // moved up
                }
                leftTmp = histogram[y];             // make current val the previous for next loop
            }
            targetInfo->edgeLeft = x;
        }

#if CPP
void ThunderVision::CalcHeightAndTopEdge(TARGET_INFO *targetInfo)
#else
        private unsafe void CalcHeightAndTopEdge(TARGET_INFO* targetInfo)
#endif
        {
            int x;
            int y;
            int topTmp = 0;
            int bottomTmp = 0;
            bool found = false;
            byte* mono;

            targetInfo->avgHeight = 0;
            targetInfo->maxHeight = 0;
            //memset(histogram, 0, sizeof(histogram));
            ClearArray(histogram);
            for (x = targetInfo->left; x <= targetInfo->right; x++)
            {
                found = false;                  // no pixels in this row yet
                fixed (byte* monoptr = &monoImage[(targetInfo->top * _imageWidth) + x])
                {
                    mono = monoptr;
                    for (y = targetInfo->top; y <= targetInfo->bottom; y++, mono += _imageWidth)
                    {
                        // if pixel is set, then update tracking variables
                        if (*mono != 0)
                        {
                            if (!found)
                            {
                                // it is the first found pixel, so save as top-most in this row
                                topTmp = y;
                                histogram[x] = y;
                            }
                            bottomTmp = y;          // any pixel with end up being the bottom-most pixel
                            found = true;           // at least one pixel
                        }
                    }
                }

                // if we found at least one pixel, then update overall tracking values
                if (found)
                {
                    // keep track of max height and the total height (so we can do average)
                    if ((bottomTmp - topTmp + 1) > targetInfo->maxHeight)
                    {
                        targetInfo->maxHeight = (bottomTmp - topTmp + 1);
                    }
                    targetInfo->avgHeight += (bottomTmp - topTmp + 1);
                }
            }

            // calc average
            targetInfo->avgHeight /= (targetInfo->right - targetInfo->left + 1);

            // determine the 'skew' of the top edge
            // we only want to look at the middle of the image (to avoid the angles of the top & bottom)
            y = 0;
            x = (targetInfo->right - targetInfo->left + 1) >> 2;    // get 1/4 of the total width
            bottomTmp = targetInfo->right - x;                      // end 1/4 from the right side
            x += targetInfo->left;                                  // start 1/4 from the left side
            topTmp = histogram[x];                                  // initialize with first value
            for (; x <= bottomTmp; x++)
            {
                // if value is greater than the previous, then the line going to right;
                // otherise the line is going to the left
                if (histogram[x] > topTmp)
                {
                    y++;                            // moved to the right
                }
                else if (histogram[x] < topTmp)
                {
                    y--;                            // moved to the left
                }
                topTmp = histogram[x];              // make current val the previous for next loop
            }
            targetInfo->edgeTop = y;
        }

        // gets the targets
        //
        // targetInfo		is a pointer to a list of targets (pass in NULL to get the total)
        // numEntires		indicates how many TARGET_INFO structures that targetInfo points to
        // returns			the number of targets copied
#if CPP
int ThunderVision::GetTargetInfo(TARGET_INFO *targetInfoList, int numEntries)
#else
        private unsafe int GetTargetInfo(TARGET_INFO* targetInfoList, int numEntries)
#endif
        {
            int i;

            // if call just wants the total, then return it
            if (targetInfoList == (TARGET_INFO*)0)
            {
                return (targets);
            }

            // copy what they want, but not more than want we have
            i = numEntries;
            if (i > targets)
            {
                i = targets;
            }
            //MABMAB memcpy((byte*)targetInfoList, (byte*)targetInfo, sizeof(TARGET_INFO) * i);
            return (i);
        }

        // create column histogram for the given target row
#if CPP
void ThunderVision::CreateRowHisto(int row)
#else
        private unsafe void CreateRowHisto(int row)
#endif
        {
            byte* mono;
            int x;
            int y;

            // reset histo
            //memset(histogram, 0, sizeof(histogram));
            ClearArray(histogram);

            // process each row within a single target row
            for (y = rowInfo[row].top; y <= rowInfo[row].bottom; y++)
            {
                // process each pixel in the row
                fixed (byte* monoptr = &monoImage[y * _imageWidth])
                {
                    mono = monoptr;
                    for (x = 0; x < _imageWidth; x++, mono++)
                    {
                        if (*mono != 0)
                        {
                            histogram[x]++; // keep track of column that has a pixel
                        }
                    }
                }
            }
        }

        // look at row's histogram and find all the targets (just getting 'width' of target)
#if CPP
void ThunderVision::FindTargets(int targetFirstInRow, int row)
#else
        private unsafe void FindTargets(int targetFirstInRow, int row)
#endif
        {
            int x;
            int i;
            int* histo;
            int gap;

            // determine how much gap between targets is allowed (i.e. ball/hoop)
            // if a target is this close to the previous target, then it will be
            // appened to the previous target
            gap = (rowInfo[row].bottom - rowInfo[row].top + 1) >> 1;    // half of the hieght

            // look at each row
            x = 0;                      // start at first col
            fixed (int* histoptr = &histogram[0])
            {
                histo = histoptr;          // start at first col (using a pointer is faster at runtime)
                while ((x < _imageWidth) && (targets < MAX_TARGETS))
                {
                    // find first pixel
                    while ((x < _imageWidth) && (*histo == 0))
                    {
                        x++;
                        histo++;
                    }
                    if (x == _imageWidth)                   // make sure we found a pixel. if not, go to next col
                    {
                        break;
                    }
                    targetInfo[targets].valid = true;                   // initialize and assume valid
                    targetInfo[targets].row = row;                      // remember row target is in
                    targetInfo[targets].left = x;                       // remember left-most col that has pixels
                    targetInfo[targets].top = rowInfo[row].top;         // remember vertical placement
                    targetInfo[targets].bottom = rowInfo[row].bottom;   // remember vertical placement
                    targetInfo[targets].avgWidth = 0;                   // 0 it since we add to this when appending

                    // find the last pixel and keep count of how many total pixels there are
                    i = 0;
                    while (x < _imageWidth && (*histo != 0))
                    {
                        i += *histo;
                        x++;
                        histo++;
                    }

#if PC_TEST_APP
                if (targetsTest < MAX_TARGETS_TEST)
                {
                    //memcpy(&targetInfoTest[targetsTest], &targetInfo[targets], sizeof(TARGET_INFO));
                    fixed (TARGET_INFO *dst = &targetInfoTest[targetsTest])
                    {
                        fixed (TARGET_INFO *src = &targetInfo[targets])
                        {
                            memcpy((byte*)dst, (byte*)src, sizeof(TARGET_INFO));
                        }
                    }
                    targetInfoTest[targetsTest].right = (x - 1);
                    targetsTest++;
                }
#endif // PC_TEST_APP

                    // if we already have a target, then see if this one is close to
                    // the previous one. this handles gaps caused by balls and hoop
                    if ((targets > targetFirstInRow)
                        && (targetInfo[targets].left < (targetInfo[targets - 1].right + gap))
                        )
                    {
                        targets--;                              // append to previous target
                    }

                    targetInfo[targets].right = (x - 1);        // remember right-most col that has pixels
                    targetInfo[targets].avgWidth += i;
                    targets++;
                }
            }

            // now that we found all the targets for this row, we need
            // to strip out any that are too small. do this pre row so we
            // don't overfill targetInfo
            //info = "";
            //info += string.Format("{0},{1}\r\n", _pixelPerRow, _minWidth);
            for (x = targetFirstInRow; x < targets; x++)
            {
#if PC_TEST_APP
                if (targetsTestG < MAX_TARGETS_TEST_G)
                {
                    //memcpy(&targetInfoTestG[targetsTestG], &targetInfo[x], sizeof(TARGET_INFO));
                    fixed (TARGET_INFO* dst = &targetInfoTestG[targetsTestG])
                    {
                        fixed (TARGET_INFO* src = &targetInfo[x])
                        {
                            memcpy((byte*)dst, (byte*)src, sizeof(TARGET_INFO));
                        }
                    }
                    targetsTestG++;
                }
#endif // PC_TEST_APP

                // to help get rid of noise and diagonal line, we will only keep
                // this set if the average number of pixels is high enough AND it is width enough
                //info += string.Format("{0},{1} {2} {3}\r\n", (targetInfo[x].avgWidth / (targetInfo[x].bottom - targetInfo[x].top + 1)), (targetInfo[x].bottom - targetInfo[x].top + 1),
                //    (((targetInfo[x].avgWidth / (targetInfo[x].bottom - targetInfo[x].top + 1)) <= 5) || ((targetInfo[x].bottom - targetInfo[x].top + 1) < 20)),
                //    (((targetInfo[x].avgWidth / (targetInfo[x].bottom - targetInfo[x].top + 1)) <= 3) || ((targetInfo[x].bottom - targetInfo[x].top + 1) < 15)));
                if (((targetInfo[x].avgWidth / (targetInfo[x].bottom - targetInfo[x].top + 1)) <= _pixelPerRow)
                    || ((targetInfo[x].bottom - targetInfo[x].top + 1) < _minWidth))
                {
                    for (i = x + 1; i < targets; i++)
                    {
                        //memcpy(&targetInfo[i - 1], &targetInfo[i], sizeof(TARGET_INFO));
                        targetInfo[i - 1] = targetInfo[i];
                    }
                    x--;
                    targets--;
                }
            }
            //System.Windows.Forms.MessageBox.Show(info, "Find");
        }

        // the following is only used by the PC test app
#if PC_TEST_APP
#if CPP
void ThunderVision::GetTargetRowImage(RGBValue *image)
#else
        private unsafe void GetTargetRowImage(RGBValue* image)
#endif
        {
            int row;
            int y;
            int x;

            for (row = 0; row < rows; row++)
            {
                for (y = rowInfo[row].top; y <= rowInfo[row].bottom; y++)
                {
                    for (x = 0; x < _imageWidth; x++)
                    {
                        // leave in original boxes
                        if (image[(y * _imageWidth) + x].G == 0)
                        {
                            // 1st row is RED, 2nd row if BLUE, 3rd row is GREEN, 4th row is WHITE, etc.
                            image[(y * _imageWidth) + x].R = (((row & 0x03) == 0) || ((row & 0x03) == 3)) ? (byte)255 : (byte)0;
                            image[(y * _imageWidth) + x].G = (((row & 0x03) == 1) || ((row & 0x03) == 3)) ? (byte)255 : (byte)0;
                            image[(y * _imageWidth) + x].B = (((row & 0x03) == 2) || ((row & 0x03) == 3)) ? (byte)255 : (byte)0;
                        }
                    }
                }
            }
        }

#if CPP
void ThunderVision::GetTargetImageWithGap(RGBValue *image)
#else
        private unsafe void GetTargetImageWithGap(RGBValue* image)
#endif
        {
            GetTargetImage(image, targetsTest, targetInfoTest);
        }
#if CPP
void ThunderVision::GetTargetImageNoGap(RGBValue *image)
#else
        private unsafe void GetTargetImageNoGap(RGBValue* image)
#endif
        {
            GetTargetImage(image, targetsTestG, targetInfoTestG);
        }
#if CPP
void ThunderVision::GetTargetImageNoNoise(RGBValue *image)
#else
        private unsafe void GetTargetImageNoNoise(RGBValue* image)
#endif
        {
            GetTargetImage(image, targets, targetInfo);
        }
#if CPP
void ThunderVision::GetTargetImageTight(RGBValue *image)
#else
        private unsafe void GetTargetImageTight(RGBValue* image)
#endif
        {
            GetTargetImage(image, targets, targetInfo);
        }
#if CPP
void ThunderVision::GetTargetImage(RGBValue *image, int maxTargets, TARGET_INFO *targetInfo)
#else
        private unsafe void GetTargetImage(RGBValue* image, int maxTargets, TARGET_INFO[] targetInfo)
#endif
        {
            int target;
            int y;
            int x;

            for (target = 0; target < maxTargets; target++)
            {
                for (y = targetInfo[target].top; y <= targetInfo[target].bottom; y++)
                {
                    for (x = targetInfo[target].left; x <= targetInfo[target].right; x++)
                    {
                        // leave in original boxes
                        if (image[(y * _imageWidth) + x].G == 0)
                        {
                            // 1st target is RED, 2nd target if BLUE, 3rd target is GREEN, 4th target is WHITE, etc.
                            image[(y * _imageWidth) + x].R = (((target & 0x03) == 0) || ((target & 0x03) == 3)) ? (byte)255 : (byte)0;
                            image[(y * _imageWidth) + x].G = (((target & 0x03) == 1) || ((target & 0x03) == 3)) ? (byte)255 : (byte)0;
                            image[(y * _imageWidth) + x].B = (((target & 0x03) == 2) || ((target & 0x03) == 3)) ? (byte)255 : (byte)0;
                        }
                    }
                }
            }
        }

#if !CPP
        private unsafe bool WriteFileTargets(string file, string tag, int width, int height)
        {
            int targets;
            string data = "";

            // get target info first
            fixed (TARGET_INFO* ti = &targetInfo[0])
            {
                targets = GetTargetInfo(ti, MAX_TARGETS);
            }

            string textFile = string.Format("{0}__{1}.txt", file, tag);
            if (file.EndsWith(".bmp"))
            {
                textFile = string.Format("{0}__{1}.txt", file.Substring(0, file.Length - 4), tag);
                file = file.Substring(0, file.Length - 4);
            }

            // print each target's info
            for (int t = 0; t < targets; t++)
            {
                data += string.Format("{0}:  r:{1}   T:{2:D3}   B:{3:D3}   L:{4:D3}   R:{5:D3}   W:{6:D3} (avg:{7:D3} max:{8:D3})   H:{9:D3} (avg:{10:D3} max:{11:D3})   E:{12:D3} {13:D3}\r\n",
                            t + 1,
                            targetInfo[t].row,
                            targetInfo[t].top,
                            targetInfo[t].bottom,
                            targetInfo[t].left,
                            targetInfo[t].right,
                            targetInfo[t].right - targetInfo[t].left + 1,
                            targetInfo[t].avgWidth,
                            targetInfo[t].maxWidth,
                            targetInfo[t].bottom - targetInfo[t].top + 1,
                            targetInfo[t].avgHeight,
                            targetInfo[t].maxHeight,
                            targetInfo[t].edgeLeft,
                            targetInfo[t].edgeTop);
            }

            if (targets == 2)
            {
                int distanceBetween = 0;
                int center = (width / 2);
                int leftCenter = 0;
                int rightCenter = 0;
                if (targetInfo[0].row != targetInfo[1].row)
                {
                    data += "not in same row\r\n";
                    targets = 0;
                }
                else if (targetInfo[0].left < targetInfo[1].left)
                {
                    distanceBetween = targetInfo[1].left - targetInfo[0].right;
                    leftCenter = targetInfo[0].left + (targetInfo[0].maxWidth / 2);
                    rightCenter = targetInfo[1].left + (targetInfo[1].maxWidth / 2);
                    center = ((rightCenter - leftCenter) / 2) + leftCenter;
                }
                else
                {
                    distanceBetween = targetInfo[0].left - targetInfo[1].right;
                }
                data += "have target\r\n";
                data += string.Format("distanceBetween {0}\r\n", distanceBetween);
                data += string.Format("l: {0} r:{1} c:{2}\r\n", leftCenter, rightCenter, center);
                data += string.Format("offcenter {0}\r\n", (center - (width / 2)));
            }

            if (targets != 2)
            {
                data += "invalidtarget";
            }

            try
            {
                System.IO.File.WriteAllText(textFile, data);
            }
            catch
            {
            }
            return (true);
        }
        private unsafe void WriteFile(string file, string tag, RGBValue *rgbValue, int width, int height)
        {
            // make sure we need to write it
            if (string.IsNullOrEmpty(file))
            {
                return;
            }

            try
            {
                using (Bitmap b = new Bitmap(width, height))
                {
                    for (int h = 0; h < height; h++)
                    {
                        for (int w = 0; w < width; w++)
                        {
                            b.SetPixel(w, h, Color.FromArgb(rgbValue->R, rgbValue->G, rgbValue->B));
                            rgbValue++;
                        }
                    }
                    b.Save(string.Format("{0}__{1}.bmp", file, tag), System.Drawing.Imaging.ImageFormat.Bmp);
                }
            }
            catch
            {
            }
        }
        private unsafe void WriteFileMono(string file, string tag, byte[] monoValue, int width, int height)
        {
            // make sure we need to write it
            if (string.IsNullOrEmpty(file))
            {
                return;
            }

            try
            {
                using (Bitmap b = new Bitmap(width, height))
                {
                    fixed (byte* monoValueptr = &monoValue[0])
                    {
                        byte* mono = monoValueptr;
                        for (int h = 0; h < height; h++)
                        {
                            for (int w = 0; w < width; w++)
                            {
                                b.SetPixel(w, h, Color.FromArgb(*mono, *mono, *mono));
                                mono++;
                            }
                        }
                        b.Save(string.Format("{0}__{1}.bmp", file, tag), System.Drawing.Imaging.ImageFormat.Bmp);
                    }
                }
            }
            catch
            {
            }
        }
        private RGBValue[] _rgbValueOrig = null;
        public unsafe bool ProcessImage(RGBValue[] rgbValue, int imageWidth, int imageHeight, string file = null)
        {
            fixed (RGBValue * rgbvalueptr = &rgbValue[0])
            {
                // STEP 1: pull out the green
                if (!ConvertRgbToMono(rgbvalueptr, imageWidth, imageHeight)) // create mono
                {
                    return (false);
                }

                // dump debug images
                if (file != null)
                {
                    // back up the original
                    _rgbValueOrig = new RGBValue[rgbValue.Length];
                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbptr[i] = rgbvalueptr[i]; } }

                    WriteFile(file, "01rgb", rgbvalueptr, imageWidth, imageHeight); // save the color version of the mono image
                    WriteFileMono(file, "02mono", monoImage, imageWidth, imageHeight); // save monochrome image
                    //System.Windows.Forms.MessageBox.Show(file, "");
                }

                // STEP 2: find the rows of the image that have targets
                CalcTargetRows();

                // dump debug image
                if (file != null)
                {
                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbvalueptr[i] = rgbptr[i]; } }
                    GetTargetRowImage(rgbvalueptr);             // fill in entire width for rows that have a target
                    WriteFile(file, "03row", rgbvalueptr, imageWidth, imageHeight);// save 'row' image
                }

                // STEP 3: find targets within each row ('width' only)
                CalcTargetColumns();

                // dump debug images
                if (file != null)
                {
                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbvalueptr[i] = rgbptr[i]; } }
                    GetTargetImageWithGap(rgbvalueptr);         // fill in with 'test' targets (with the gaps)
                    WriteFile(file, "04gap", rgbvalueptr, imageWidth, imageHeight);// save 'with gap' image

                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbvalueptr[i] = rgbptr[i]; } }
                    GetTargetImageNoGap(rgbvalueptr);           // fill in with 'col' targets (without the gaps)
                    WriteFile(file, "05nog", rgbvalueptr, imageWidth, imageHeight);// save 'no gap' image

                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbvalueptr[i] = rgbptr[i]; } }
                    GetTargetImageNoNoise(rgbvalueptr);         // fill in with 'col' targets (without the noise)
                    WriteFile(file, "06non", rgbvalueptr, imageWidth, imageHeight);// save 'no noise' image
                }

                // STEP 4: fine tune targets
                CalcTargets();

                // dump debug images
                if (file != null)
                {
                    fixed (RGBValue* rgbptr = &_rgbValueOrig[0]) { for (int i = 0; i < rgbValue.Length; i++) { rgbvalueptr[i] = rgbptr[i]; } }
                    GetTargetImageTight(rgbvalueptr);           // fill in with 'col' targets (tight box)
                    WriteFile(file, "final", rgbvalueptr, imageWidth, imageHeight);// save 'no noise' image
                    WriteFileTargets(file, "99inf", imageWidth, imageHeight);
                }
                return (targets != 0);
	        }
        }
#endif //!CPP
#endif // PC_TEST_APP
#if !CPP
    }
}
#endif
